package geniusweb.protocol.partyconnection;

import java.io.IOException;
import java.util.List;

import geniusweb.actions.Action;
import geniusweb.connection.ConnectionFactory;
import geniusweb.inform.Inform;
import geniusweb.references.Reference;
import tudelft.utilities.repository.NoResourcesNowException;

/**
 * An extended {@link ConnectionFactory} that store PartyIDs with the
 * connections, making it possible to find back the appropriate connection given
 * a PartyID. Also, it allows the connection factory to actually assign the
 * PartyIDs.
 *
 */
public interface ProtocolToPartyConnFactory
		extends ConnectionFactory<Action, Inform> {
	@Override
	public ProtocolToPartyConn connect(Reference reference)
			throws IOException, NoResourcesNowException;

	/**
	 * Connects a set of References in one go, following a modified Banker's
	 * algorithm. Only succeeds if all requested connections can be made.
	 * 
	 * @param references the list of references to connect with
	 * @return a List of connections, one fore each Reference. The URI Scheme
	 *         deteremines how the connection to the party is created.
	 *         Typically,
	 *         <ul>
	 *         <li>"classpath" assumes the file refers to an existing class on
	 *         the classpath, directly to be instantiated.
	 *         <li>"pythonpath" idem, but now the party is a python party.
	 *         <li>"http" assumes the party is to be started on a partiesserver,
	 *         where the partiesserver will convert the provided address to a
	 *         specific party.
	 *         </ul>
	 *         It depends on the run system which of these protocols are
	 *         actually supported, eg "pythonpath" is typically only supported
	 *         for the python-based simplerunner.
	 * @throws IOException             if a connection is failing irrecoverably.
	 * @throws NoResourcesNowException is there are insufficient resources now.
	 *                                 This exception includes a suggested
	 *                                 moment to retry.
	 */
	List<ProtocolToPartyConn> connect(List<Reference> references)
			throws IOException, NoResourcesNowException;

}
